// Copyright 2026 Tencent
// SPDX-License-Identifier: BSD-3-Clause

#include "cpu.h"
#include "mat.h"
#include "x86_usability.h"

namespace ncnn {

#include "groupnorm_bf16s.h"

void groupnorm_bf16s_sse_avx512bf16(unsigned short* ptr, const float* gamma_ptr, const float* beta_ptr, float eps, int channels, int size, int elempack, size_t cstep)
{
    groupnorm_bf16s_sse(ptr, gamma_ptr, beta_ptr, eps, channels, size, elempack, cstep);
}

} // namespace ncnn
